#
# Wiki - ɵWiki
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: TsuWiki.pm,v 1.8 2004/06/01 12:51:53 fuktommy Exp $
#
package TsuWiki;
use strict;

require Exporter;
use vars qw(@ISA @EXPORT $DATA_DIR $SAVE_HISTORY $ART_LIMIT);
@ISA = qw(Exporter);
@EXPORT = qw(
	str_encode escape escapeValue unescape args rec
	readFile margeRecord addRecord list stamp setStamp search
);

# ѿϸƽФ¦ꤹ뤳ȤꤷƤ
$DATA_DIR	= "./data";		# ǡ񤭹Ȥ
$SAVE_HISTORY	= 10;			# Խ򲿸Ĥޤ¸뤫
$ART_LIMIT	= 10240;		# 񤭹ߤκ祵(Bytes)

#
# lock
#
sub lock() {
	open LOCK, "> $DATA_DIR/lock";
	flock LOCK, 002;
}
sub unlock() {
	close LOCK;
}


#
# encode/decode
#
sub escape($) {
	local($_) = @_;
	s/&/&amp;/g;
	s/&amp;(#\d+|#[Xx][0-9A-Fa-f]+|[A-Za-z0-9]+);/&$1;/g;
	s/</&lt;/g;
	s/>/&gt;/g;
	s/\r//g;
	s/\n/<br>/g;
	return $_;
}
sub escapeValue($) {
	local($_) = @_;
	s/"/&quot;/g;
	return $_;
}
sub unescape($) {
	local($_) = @_;
	s|<br>|\n|g;
	return $_;
}
sub str_encode($) {		# Thanks for YukiWiki & WalWiki
	my($s) = @_;
	$s =~ s/[^-\w]/'%' . uc(unpack('H2', $&))/eg;
	return $s;
}
sub str_decode($) {		# Thanks for YukiWiki
	my($s) = @_;
	$s =~ s/\+/ /g;
	$s =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C", hex($1))/eg;
	return escape($s);
}
sub file_encode($) {		# Thanks for YukiWiki
	my($s) = @_;
	$s =~ s/./uc(unpack('H2', $&))/eg;
	return $s;
}
sub file_decode($) {		# Thanks for YukiWiki
	my($s) = @_;
	$s =~ s/[0-9A-F][0-9A-F]/pack("C", hex($&))/eg;
	return escape($s);
}
sub rec($) {
	local($_) = @_;
	chomp;
	@_ = split /<>/, $_, 2;
	my %buf = (stamp=>$_[0], body=>$_[1]);
	return %buf;
}

#
# β
#
sub args($) {
	local $_ = $_[0];
	local @_ = split /&/;
	my %arg = ();
	foreach (@_) {
		local @_ = split /=/;
		if (defined $_[1]) {
			$arg{$_[0]} = str_decode($_[1]);
		} else {
			$arg{$_[0]} = "";
		}
	}
	if (keys %arg == 1) {
		my @k = keys %arg;
		if ($arg{$k[0]} eq "") {
			$arg{cmd} = "view";
			$arg{msg} = str_decode($k[0]);
		}
	}
	return %arg;
}

#
# եɤ߹
#
sub readFile($) {
	my($file) = @_;
	my @buf = ();
	$file = file_encode($file);
	if (open IN, "$DATA_DIR/$file.txt") {
		@buf = <IN>;
		chomp @buf;
		close IN;
	}
	return @buf;
}

#
# եԽƤɵ
#
sub addRecord($$$) {
	my($file, $record, $base) = @_;
	my @buf = readFile($file);
	$file = file_encode($file);
	my $time = time;
	if (@buf && ($buf[-1] !~ /^$base<>/)) {
		return 0;
	} elsif (length $record > $ART_LIMIT) {
		return 0;
	} elsif (@buf && ($buf[-1] =~ /^\d+<>$/) && ($record eq "")) {
		unlink "$DATA_DIR/$file.txt";
		return 1;
	}
	push @buf, "$time<>$record";
	while (@buf > $SAVE_HISTORY) {
		shift @buf;
	}
	lock();
	if (open OUT, "> $DATA_DIR/$file.txt") {
		local $_;
		foreach (@buf) {
			print OUT "$_\n";
		}
		close OUT;
		unlock();
		return 1;
	} else {
		unlock();
		return 0;
	}
}

#
# եɵ
#
sub margeRecord($$) {
	my($file, $record) = @_;
	my @buf = readFile($file);
	my $base = "";
	if (@buf) {
		my %rec = rec($buf[-1]);
		my $time = time;
		if ($rec{body} =~ /<br>$/) {
			$record = "$rec{body}<br>$record";
		} else {
			$record = "$rec{body}<br><br>$record";
		}
		$base = $rec{stamp};
	}
	return addRecord($file, $record, $base);
}

#
# ե
#
sub list() {
	local $_;
	my @buf = ();
	foreach (glob "$DATA_DIR/*.txt") {
		s|$DATA_DIR/||;
		s|\.txt$||;
		push @buf, file_decode($_);
	}
	return @buf;
}

#
# 
#
sub stamp($) {
	my($file) = @_;
	$file = "$DATA_DIR/" . file_encode($file) . ".txt";
	my @stat = stat $file;
	return $stat[9];
}

#
# 
#
sub setStamp($$) {
	my($file, $stamp) = @_;
	$file = "$DATA_DIR/" . file_encode($file) . ".txt";
	utime $stamp, $stamp, $file;
}

#
# 
#
sub search($) {
	my($query) = @_;
	my $file;
	my @result = ();
	foreach $file (glob("$DATA_DIR/*.txt")) {
		my $file2 = $file;
		$file2 =~ s|^$DATA_DIR/||; $file2 =~ s|\.txt$||;
		my $x = file_decode($file2);
		my $found;
		if ($x =~ /$query/) {
			$found = 1;
		} else {
			open IN, $file or next;
			while (<IN>) {
				if (/\Q$query\E/) {
					$found = 1;
					last;
				}
			}
			close IN;
		}
		if ($found) {
			push @result, $x;
		}
	}
	return @result;
}

1;
